/*
  Top10, a racing simulator
  Copyright (C) 2000-2004  Johann Deneux
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Authors can be contacted at following electronic addresses:
  Johann Deneux: johann.deneux@it.uu.se
*/

#include "PathFinder.hh"
#include <cstdlib>
#include <sys/types.h>
#include <sys/stat.h>

using namespace top10;
using namespace top10::util;

std::list<std::string> top10::util::PathFinder::paths;

std::string PathFinder::find(std::string filename)
{
  struct stat buf;

  std::string name(filename);
  if (isAbsolute(name)) {
    if (stat(name.c_str(), &buf) == 0) return name;
    name = strip(name);
  }

  while (!name.empty()) {
    for (Paths::const_iterator path = paths.begin();
	 path != paths.end();
	 ++path) {
      std::string tmp = *path + std::string("/") + name;
      if (stat(tmp.c_str(), &buf) == 0) return tmp;
    }
    name = strip(name);
  }

  return name;  // name = ""
}

bool PathFinder::isAbsolute(std::string name)
{
  return '/' == name[0];
}

std::string PathFinder::strip(std::string name)
{
  std::string ret;

  // Look for the first '/', and remove it
  std::string::size_type pos = name.find('/', 0);
  if (pos != std::string::npos)
    ret = name.substr(pos+1, name.size() - pos -1);

  return ret;
}

void PathFinder::addPath(std::string new_path)
{
  for (Paths::const_iterator path = paths.begin();
       path != paths.end();
       ++path)
    if (*path == new_path) return;

  paths.push_back(new_path);
}
